@@ -1,3 +1,4 @@
+{-# OPTIONS_GHC -Wno-orphans #-}
 {-# LANGUAGE UndecidableInstances #-}
 module Darcs.Test.Patch.Arbitrary.RepoPatchV2 () where
 
@@ -6,13 +7,16 @@ import Darcs.Prelude
 import Control.Exception
 import System.IO.Unsafe
 
-import Darcs.Test.Patch.Arbitrary.Generic ( MightHaveDuplicate(..), PrimBased(..), ArbitraryPrim )
+import Darcs.Test.Patch.Arbitrary.Generic
+    ( ArbitraryPrim
+    , MightHaveDuplicate(..)
+    , PrimBased(..)
+    )
 import Darcs.Test.Patch.Arbitrary.RepoPatch
 import Darcs.Test.Patch.Merge.Checked ( CheckedMerge(..) )
 import Darcs.Test.Patch.RepoModel ( RepoState, ModelOf )
 import Darcs.Test.Patch.WithState ( PropagateShrink )
 import Darcs.Patch
-import Darcs.Patch.Annotate
 import Darcs.Patch.V2 ( RepoPatchV2 )
 import Darcs.Patch.V2.RepoPatch ( isDuplicate, RepoPatchV2(Normal) )
 import Darcs.Patch.Witnesses.Ordered
@@ -22,12 +26,12 @@ instance MightHaveDuplicate (RepoPatchV2 prim) where
 
 type instance ModelOf (RepoPatchV2 prim) = ModelOf prim
 
-instance
-  (Annotate prim, ArbitraryPrim prim, PrimPatch prim, ApplyState prim ~ RepoState (ModelOf prim))
-  => ArbitraryRepoPatch (RepoPatchV2 prim)
-  where
-
-    notRepoPatchV1 = Just (NotRepoPatchV1 (\case {}))
+instance ( ArbitraryPrim prim
+         , PrimPatch prim
+         , ApplyState prim ~ RepoState (ModelOf prim)
+         ) =>
+         ArbitraryRepoPatch (RepoPatchV2 prim) where
+  notRepoPatchV1 = Just (NotRepoPatchV1 (\case {}))
 
 instance PrimPatch prim => CheckedMerge (RepoPatchV2 prim) where
   validateMerge v =
@@ -35,7 +39,8 @@ instance PrimPatch prim => CheckedMerge (RepoPatchV2 prim) where
       Left (_ :: SomeException) -> Nothing
       Right x -> Just x
 
-instance (PrimPatch prim, ArbitraryPrim prim, PropagateShrink prim prim) => PrimBased (RepoPatchV2 prim) where
+instance (PrimPatch prim, ArbitraryPrim prim, PropagateShrink prim prim) =>
+         PrimBased (RepoPatchV2 prim) where
   type OnlyPrim (RepoPatchV2 prim) = prim
   primEffect = (:>: NilFL)
   liftFromPrim = Normal
