@@ -28,6 +28,7 @@ import qualified Darcs.Patch.Prim as W ( coalesce )
 import qualified Darcs.Patch.Witnesses.Ordered as W
 import Darcs.Patch.Witnesses.Sealed
 import Darcs.Patch.Witnesses.Eq
+import Darcs.Patch.Witnesses.Maybe
 import Darcs.Patch.Witnesses.Show
 import Darcs.Patch.Witnesses.Unsafe ( unsafeCoerceP, unsafeCoercePStart, unsafeCoercePEnd )
 
@@ -135,5 +136,4 @@ getTriples :: FL (RepoPatchV2 Prim2) wX wY -> [Sealed2 (RepoPatchV2 Prim2 :> Rep
 getTriples = map (mapSeal2 fromW) . W.getTriples . toW
 
 coalesce :: (Prim2 :> Prim2) wX wY -> Maybe (FL Prim2 wX wY)
-coalesce = fmap fromW . W.coalesce . toW
-
+coalesce = fmap (fromW . maybeToFL) . W.coalesce . toW
