@@ -72,18 +72,16 @@ module Darcs.Patch.Choices
     , getLabelInt
     ) where
 
-import Prelude ()
 import Darcs.Prelude
 
-import Darcs.Patch.Merge ( Merge, merge )
 import Darcs.Patch.Invert ( Invert, invert )
 import Darcs.Patch.Commute ( Commute, commute, commuteRL )
 import Darcs.Patch.Inspect ( PatchInspect, listTouchedFiles, hunkMatches )
 import Darcs.Patch.Permutations ( commuteWhatWeCanRL, commuteWhatWeCanFL )
-import Darcs.Patch.Witnesses.Eq ( Eq2(..), EqCheck(..) )
+import Darcs.Patch.Witnesses.Eq ( EqCheck(..) )
 import Darcs.Patch.Witnesses.Ordered
     ( FL(..), RL(..)
-    , (:>)(..), (:\/:)(..), (:/\:)(..), (:||:)(..)
+    , (:>)(..), (:||:)(..)
     , zipWithFL, mapFL_FL, concatFL
     , (+>+), reverseRL, anyFL )
 import Darcs.Patch.Witnesses.Sealed ( Sealed2(..) )
@@ -150,9 +148,6 @@ unLabel (LP _ p) = p
 compareLabels :: LabelledPatch p wA wB -> LabelledPatch p wC wD -> EqCheck (wA, wB) (wC, wD)
 compareLabels (LP l1 _) (LP l2 _) = if l1 == l2 then unsafeCoerceP IsEq else NotEq
 
-instance Eq2 p => Eq2 (LabelledPatch p) where
-  unsafeCompare (LP l1 p1) (LP l2 p2) = l1 == l2 && unsafeCompare p1 p2
-
 instance Invert p => Invert (LabelledPatch p) where
   invert (LP t p) = LP t (invert p)
 
@@ -165,11 +160,6 @@ instance PatchInspect p => PatchInspect (LabelledPatch p) where
   listTouchedFiles = listTouchedFiles . unLabel
   hunkMatches f = hunkMatches f . unLabel
 
-instance Merge p => Merge (LabelledPatch p) where
-  merge (LP l1 p1 :\/: LP l2 p2) =
-    case merge (p1 :\/: p2) of
-      p2' :/\: p1' -> LP l2 p2' :/\: LP l1 p1'
-
 instance Commute p => Commute (PatchChoice p) where
   commute (PC p1 c1 :> PC p2 c2) = do
     p2' :> p1' <- commute (p1 :> p2)
@@ -179,10 +169,6 @@ instance PatchInspect p => PatchInspect (PatchChoice p) where
   listTouchedFiles = listTouchedFiles . pcPatch
   hunkMatches f = hunkMatches f . pcPatch
 
-instance Merge p => Merge (PatchChoice p) where
-  merge (PC lp1 c1 :\/: PC lp2 c2) = case merge (lp1 :\/: lp2) of
-    lp2' :/\: lp1' -> PC lp2' c2 :/\: PC lp1' c1
-
 -- | Create a 'PatchChoices' from a sequence of patches, so that
 -- all patches are initially 'InMiddle'.
 patchChoices :: FL p wX wY -> PatchChoices p wX wY
@@ -197,9 +183,6 @@ labelPatches tg ps = zipWithFL LP (map (Label tg) [1..]) ps
 mkPatchChoices :: FL (LabelledPatch p) wX wY -> PatchChoices p wX wY
 mkPatchChoices = PCs NilFL . mapFL_FL (pcSetLast False)
 
-instance Eq2 p => Eq2 (PatchChoice p) where
-  unsafeCompare (PC lp1 _) (PC lp2 _) = unsafeCompare lp1 lp2
-
 
 -- | Like 'getChoices' but lumps together 'InMiddle' and 'InLast' patches.
 -- This is more efficient than using 'getChoices' and then catenating 'InMiddle'
@@ -309,7 +292,7 @@ patchSlot (LP t _) pc@(PCs f ml)
       case commuteRL (bubble :> lp) of
         Just (lp' :> bubble') -> psLast firsts (middles :<: lp') bubble' ls
         Nothing -> psLast firsts middles (bubble :<: lp) ls
-    psLast _ _ _ NilFL = impossible
+    psLast _ _ _ NilFL = error "impossible case"
     settleM middles = mapFL_FL (\lp -> PC lp False) $ reverseRL middles
     settleB bubble = mapFL_FL (\lp -> PC lp True) $ reverseRL bubble
 
