@@ -29,7 +29,9 @@ instance PrimRead Prim where
       identity = lexString "identity" >> return Identity
       patch x = string x >> uuid
       uuid = UUID <$> lexWord
-      filename = decodeWhiteName <$> lexWord
+      filename = do
+        word <- lexWord
+        either fail return $ decodeWhiteName word
       content = do
         lexString "content"
         len <- int
