@@ -1,5 +1,5 @@
 {-# OPTIONS_GHC -fno-warn-orphans #-}
-{-# LANGUAGE ViewPatterns #-}
+{-# LANGUAGE ViewPatterns, UndecidableInstances #-}
 module Darcs.Patch.Prim.V1.Show
     ( showHunk )
     where
@@ -8,9 +8,8 @@ import Darcs.Prelude
 
 import Darcs.Util.ByteString ( fromPS2Hex )
 import qualified Data.ByteString as B (ByteString, length, take, drop)
-import qualified Data.ByteString.Char8 as BC (head)
 
-import Darcs.Patch.Apply ( ApplyState )
+import Darcs.Patch.Apply ( Apply(..), ObjectIdOfPatch )
 import Darcs.Patch.FileHunk ( FileHunk(..), showFileHunk )
 import Darcs.Patch.Format ( FileNameFormat )
 import Darcs.Patch.Show ( formatFileName )
@@ -27,42 +26,17 @@ import Darcs.Util.Printer ( Doc, vcat,
                  text, userchunk, invisibleText, invisiblePS, blueText,
                  ($$), (<+>)
                )
-import Darcs.Util.Show ( appPrec, BSWrapper(..) )
 import Darcs.Util.Tree ( Tree )
 
 
-deriving instance Show (Prim wX wY)
-
 instance Show2 Prim
-
 instance Show1 (Prim wX)
-
-instance Show (FilePatchType wX wY) where
-    showsPrec _ RmFile = showString "RmFile"
-    showsPrec _ AddFile = showString "AddFile"
-    showsPrec d (Hunk line old new) | all ((==1) . B.length) old && all ((==1) . B.length) new
-        = showParen (d > appPrec) $ showString "Hunk " .
-                                      showsPrec (appPrec + 1) line . showString " " .
-                                      showsPrecC old . showString " " .
-                                      showsPrecC new
-       where showsPrecC [] = showString "[]"
-             showsPrecC ss = showParen True $ showString "packStringLetters " . showsPrec (appPrec + 1) (map BC.head ss)
-    showsPrec d (Hunk line old new) = showParen (d > appPrec) $ showString "Hunk " .
-                                      showsPrec (appPrec + 1) line . showString " " .
-                                      showsPrec (appPrec + 1) (map BSWrapper old) . showString " " .
-                                      showsPrec (appPrec + 1) (map BSWrapper new)
-    showsPrec d (TokReplace t old new) = showParen (d > appPrec) $ showString "TokReplace " .
-                                         showsPrec (appPrec + 1) t . showString " " .
-                                         showsPrec (appPrec + 1) old . showString " " .
-                                         showsPrec (appPrec + 1) new
-    -- this case may not work usefully
-    showsPrec d (Binary old new) = showParen (d > appPrec) $ showString "Binary " .
-                                   showsPrec (appPrec + 1) (BSWrapper old) . showString " " .
-                                   showsPrec (appPrec + 1) (BSWrapper new)
-
+deriving instance Show (Prim wX wY)
+deriving instance Show (FilePatchType wX wY)
 deriving instance Show (DirPatchType wX wY)
 
-instance ApplyState Prim ~ Tree => PrimShow Prim where
+instance (Apply Prim, ApplyState Prim ~ Tree, ObjectIdOfPatch Prim ~ AnchoredPath) =>
+         PrimShow Prim where
   showPrim fmt (FP f AddFile) = showAddFile fmt f
   showPrim fmt (FP f RmFile)  = showRmFile fmt f
   showPrim fmt (FP f (Hunk line old new))  = showHunk fmt f line old new
@@ -72,8 +46,13 @@ instance ApplyState Prim ~ Tree => PrimShow Prim where
   showPrim fmt (DP d RmDir)  = showRmDir fmt d
   showPrim fmt (Move f f') = showMove fmt f f'
   showPrim _ (ChangePref p f t) = showChangePref p f t
-  showPrimCtx fmt (FP f (Hunk line old new)) = showContextHunk fmt (FileHunk f line old new)
-  showPrimCtx fmt p = return $ showPrim fmt p
+  showPrimWithContextAndApply fmt p@(FP f (Hunk line old new)) = do
+    r <- showContextHunk fmt (FileHunk f line old new)
+    apply p
+    return r
+  showPrimWithContextAndApply fmt p = do
+    apply p
+    return $ showPrim fmt p
 
 showAddFile :: FileNameFormat -> AnchoredPath -> Doc
 showAddFile fmt f = blueText "addfile" <+> formatFileName fmt f
