@@ -27,9 +27,10 @@ module Darcs.Patch.Read
 import Darcs.Prelude
 
 import Control.Applicative ( (<|>) )
-import Control.Monad ( mzero )
-import qualified Data.ByteString as B ( ByteString, null )
+import Control.Monad ( mzero, (<=<) )
+import qualified Data.ByteString as B ( ByteString )
 import qualified Data.ByteString.Char8 as BC ( ByteString, pack, stripPrefix )
+import GHC.Stack ( HasCallStack )
 
 import Darcs.Patch.Bracketed ( Bracketed(..), unBracketedFL )
 import Darcs.Patch.Format
@@ -45,11 +46,12 @@ import Darcs.Util.Parser
     , lexString
     , lexWord
     , parse
+    , parseAll
     )
 import Darcs.Patch.Witnesses.Ordered ( FL(..), RL, reverseFL )
 import Darcs.Patch.Witnesses.Sealed ( Sealed(..), mapSeal )
 
-import Darcs.Util.ByteString ( decodeLocale, dropSpace, unpackPSFromUTF8 )
+import Darcs.Util.ByteString ( decodeLocale, unpackPSFromUTF8 )
 import Darcs.Util.Path ( AnchoredPath, decodeWhite, floatPath )
 
 -- | This class is used to decode patches from their binary representation.
@@ -60,12 +62,7 @@ readPatchPartial :: ReadPatch p => B.ByteString -> Either String (Sealed (p wX),
 readPatchPartial = parse readPatch'
 
 readPatch :: ReadPatch p => B.ByteString -> Either String (Sealed (p wX))
-readPatch ps =
-  case parse readPatch' ps of
-    Left e -> Left e
-    Right (p, leftover)
-      | B.null (dropSpace leftover) -> Right p
-      | otherwise -> Left $ unlines ["leftover:",show leftover]
+readPatch = parseAll readPatch'
 
 instance ReadPatch p => ReadPatch (Bracketed p) where
     readPatch' = mapSeal Braced <$> bracketedFL readPatch' '{' '}'
@@ -123,15 +120,18 @@ peekfor ps ifstr ifnot = choice [ do lexString ps
 {-# INLINE peekfor #-}
 
 -- See also Darcs.Patch.Show.formatFileName.
-readFileName :: FileNameFormat -> Parser AnchoredPath
+readFileName :: HasCallStack => FileNameFormat -> Parser AnchoredPath
 readFileName fmt = do
   raw <- lexWord
   case BC.stripPrefix (BC.pack "./") raw of
     Nothing -> fail $ "invalid file path"
-    Just raw' -> return $ convert fmt raw'
+    Just raw' ->
+      case convert fmt raw' of
+        Left e -> fail e
+        Right r -> return r
   where
     convert FileNameFormatV1 =
-      floatPath . decodeWhite . decodeLocale . BC.pack . unpackPSFromUTF8
+      floatPath <=< decodeWhite . decodeLocale . BC.pack . unpackPSFromUTF8
     convert FileNameFormatV2 =
-      floatPath . decodeWhite . decodeLocale
+      floatPath <=< decodeWhite . decodeLocale
     convert FileNameFormatDisplay = error "readFileName called with FileNameFormatDisplay"
