@@ -21,9 +21,11 @@ module Darcs.UI.Commands
     , DarcsCommand(..)
     , commandAlias
     , commandStub
-    , commandOptions
-    , commandAlloptions
     , withStdOpts
+    , commandOptDescr
+    , commandAlloptions
+    , commandDefaults
+    , commandCheckOptions
     , disambiguateCommands
     , CommandArgs(..)
     , getSubcommands
@@ -51,6 +53,7 @@ module Darcs.UI.Commands
 
 import Control.Monad ( when, unless )
 import Data.List ( sort, isPrefixOf )
+import Data.Maybe ( maybeToList )
 import System.Console.GetOpt ( OptDescr )
 import System.IO ( stderr )
 import System.IO.Error ( catchIOError )
@@ -68,15 +71,26 @@ import Darcs.Patch.Witnesses.Ordered ( FL, mapFL )
 import qualified Darcs.Repository as R ( amInHashedRepository, amInRepository
                                        , amNotInRepository, findRepository )
 import Darcs.Repository.Flags ( WorkRepo(..) )
-import Darcs.Repository.Prefs ( defaultrepo )
+import Darcs.Repository.Prefs ( getDefaultRepo )
 
-import Darcs.UI.Options ( DarcsOption, DarcsOptDescr, (^), optDescr, odesc, parseFlags, (?) )
+import Darcs.UI.Options
+    ( DarcsOptDescr
+    , DarcsOption
+    , OptMsg
+    , defaultFlags
+    , ocheck
+    , odesc
+    , optDescr
+    , parseFlags
+    , (?)
+    , (^)
+    )
 import Darcs.UI.Options.All
     ( StdCmdAction, stdCmdActions, debugging, UseCache, useCache, HooksConfig, hooks
-    , Verbosity(..), DryRun(..), dryRun, newRepo, verbosity
+    , Verbosity(..), DryRun(..), dryRun, newRepo, verbosity, UseIndex, useIndex, yes
     )
 
-import Darcs.UI.Flags ( DarcsFlag, remoteRepos, workRepo, quiet, verbose )
+import Darcs.UI.Flags ( DarcsFlag, workRepo, quiet, verbose )
 import Darcs.UI.External ( viewDoc )
 import Darcs.UI.PrintPatch ( showWithSummary )
 
@@ -133,10 +147,7 @@ data DarcsCommand =
                                 -> [DarcsFlag] -> [String] -> IO [String]
           , commandArgdefaults :: [DarcsFlag] -> AbsolutePath -> [String]
                                -> IO [String]
-          , commandBasicOptions :: [DarcsOptDescr DarcsFlag]
-          , commandAdvancedOptions :: [DarcsOptDescr DarcsFlag]
-          , commandDefaults :: [DarcsFlag]
-          , commandCheckOptions :: [DarcsFlag] -> [String]
+          , commandOptions :: CommandOptions
           }
     | SuperCommand
           { commandProgramName
@@ -147,25 +158,55 @@ data DarcsCommand =
           , commandSubCommands :: [CommandControl]
           }
 
-withStdOpts :: DarcsOption (Maybe StdCmdAction -> Verbosity -> b) c
-            -> DarcsOption (UseCache -> HooksConfig -> Bool -> Bool -> Bool -> a) b
-            -> DarcsOption a c
-withStdOpts basicOpts advancedOpts =
-  basicOpts ^ stdCmdActions ^ verbosity ^ advancedOpts ^ useCache ^ hooks ^ debugging
+data CommandOptions = CommandOptions
+  { coBasicOptions :: [DarcsOptDescr DarcsFlag]
+  , coAdvancedOptions :: [DarcsOptDescr DarcsFlag]
+  , coDefaults :: [DarcsFlag]
+  , coCheckOptions :: [DarcsFlag] -> [OptMsg]
+  }
 
-commandAlloptions :: DarcsCommand -> ([DarcsOptDescr DarcsFlag], [DarcsOptDescr DarcsFlag])
-commandAlloptions DarcsCommand { commandBasicOptions = opts1
-                               , commandAdvancedOptions = opts2 } =
-    ( opts1 ++ odesc stdCmdActions
-    , odesc verbosity ++ opts2 ++ odesc useCache ++ odesc hooks ++ odesc debugging
-    )
-commandAlloptions SuperCommand { } = (odesc stdCmdActions, [])
+-- | Construct 'CommandOptions' from the command specific basic and advanced
+-- 'DarcsOption's
+withStdOpts
+  :: DarcsOption (Maybe StdCmdAction -> Verbosity -> b) c
+  -> DarcsOption
+      (UseCache -> UseIndex -> HooksConfig -> Bool -> Bool -> [DarcsFlag]) b
+  -> CommandOptions
+withStdOpts bopts aopts =
+  CommandOptions
+    { coBasicOptions = odesc bopts'
+    , coAdvancedOptions = odesc aopts'
+    , coDefaults = defaultFlags opts
+    , coCheckOptions = ocheck opts
+    }
+  where
+    aopts' = verbosity ^ aopts ^ useCache ^ useIndex ^ hooks ^ debugging
+    bopts' = bopts ^ stdCmdActions
+    opts = bopts' ^ aopts'
 
---  Obtain options suitable as input to System.Console.Getopt, including the
---  --disable option (which is not listed explicitly in the DarcsCommand
---  definitions).
-commandOptions :: AbsolutePath -> DarcsCommand -> [OptDescr DarcsFlag]
-commandOptions cwd = map (optDescr cwd) . uncurry (++) . commandAlloptions
+-- | For the given 'DarcsCommand' check the given 'DarcsFlag's for
+-- consistency
+commandCheckOptions :: DarcsCommand -> [DarcsFlag] -> [OptMsg]
+commandCheckOptions DarcsCommand {commandOptions=co} = coCheckOptions co
+commandCheckOptions SuperCommand {} = ocheck stdCmdActions
+
+-- | Built-in default values for all 'DarcsFlag's supported by the given
+-- command
+commandDefaults :: DarcsCommand -> [DarcsFlag]
+commandDefaults DarcsCommand {commandOptions=co} = coDefaults co
+commandDefaults SuperCommand {} = defaultFlags stdCmdActions
+
+-- | Option descriptions split into basic and advanced options
+commandAlloptions :: DarcsCommand
+                  -> ([DarcsOptDescr DarcsFlag], [DarcsOptDescr DarcsFlag])
+commandAlloptions DarcsCommand {commandOptions = co} =
+  (coBasicOptions co, coAdvancedOptions co)
+commandAlloptions SuperCommand {} = (odesc stdCmdActions, [])
+
+-- | Option descriptions as required by 'System.Console.Getopt.getOpt',
+-- i.e. resolved with the given 'AbsolutePath'.
+commandOptDescr :: AbsolutePath -> DarcsCommand -> [OptDescr DarcsFlag]
+commandOptDescr cwd = map (optDescr cwd) . uncurry (++) . commandAlloptions
 
 nodefaults :: [DarcsFlag] -> AbsolutePath -> [String] -> IO [String]
 nodefaults _ _ = return
@@ -193,11 +234,15 @@ commandAlias alias msuper command =
     cmdName = unwords . map commandName . maybe id (:) msuper $ [command]
 
 commandStub :: String -> Doc -> String -> DarcsCommand -> DarcsCommand
-commandStub n h d c = c { commandName = n
-                        , commandHelp = h
-                        , commandDescription = d
-                        , commandCommand = \_ _ _ -> viewDoc h
-                        }
+commandStub n h d command@DarcsCommand {} =
+  command
+    { commandName = n
+    , commandHelp = h
+    , commandDescription = d
+    , commandCommand = \_ _ _ -> viewDoc h
+    }
+commandStub _ _ _ SuperCommand {} =
+  error "commandStub called with SuperCommand argument"
 
 superName :: Maybe (DarcsCommand) -> String
 superName Nothing  = ""
@@ -241,6 +286,7 @@ putInfo flags = unless (quiet flags) . putDocLnWith fancyPrinters
 
 putFinished :: [DarcsFlag] -> String -> IO ()
 putFinished flags what =
+  unless (yes (dryRun ? flags)) $
     putInfo flags $ "Finished" <+> text what <> "."
 
 putWarning :: [DarcsFlag] -> Doc -> IO ()
@@ -256,7 +302,7 @@ abortRun flags msg = if parseFlags dryRun flags == YesDryRun
 
 -- | Set the DARCS_PATCHES and DARCS_PATCHES_XML environment variables with
 -- info about the given patches, for use in post-hooks.
-setEnvDarcsPatches :: RepoPatch p => FL (PatchInfoAnd rt p) wX wY -> IO ()
+setEnvDarcsPatches :: RepoPatch p => FL (PatchInfoAnd p) wX wY -> IO ()
 setEnvDarcsPatches ps = do
     let k = "Defining set of chosen patches"
     let filepaths = map (anchorPath ".") (listTouchedFiles ps)
@@ -282,10 +328,12 @@ setEnvDarcsFiles ps = do
     setEnvCautiously "DARCS_FILES" $ unlines filepaths
 
 -- | Set some environment variable to the given value, unless said value is
--- longer than 10K characters, in which case do nothing.
+-- longer than 100K characters, in which case do nothing.
 setEnvCautiously :: String -> String -> IO ()
 setEnvCautiously e v
-    | toobig (10 * 1024) v = return ()
+    | toobig (100 * 1024) v =
+        hPutDocLn stderr $ text $
+          "Warning: not setting env var " ++ e ++ " (would exceed 100K)"
     | otherwise =
         setEnv e v `catchIOError` (\_ -> setEnv e (decodeLocale (packStringToUTF8 v)))
   where
@@ -295,8 +343,10 @@ setEnvCautiously e v
     toobig _ [] = False
     toobig n (_ : xs) = toobig (n - 1) xs
 
+-- | To use for commandArgdefaults field.
 defaultRepo :: [DarcsFlag] -> AbsolutePath -> [String] -> IO [String]
-defaultRepo fs = defaultrepo (remoteRepos ? fs)
+defaultRepo _ _ [] = maybeToList <$> getDefaultRepo
+defaultRepo _ _ args = return args
 
 amInHashedRepository :: [DarcsFlag] -> IO (Either String ())
 amInHashedRepository fs = R.amInHashedRepository (workRepo fs)
