@@ -45,7 +45,7 @@ import Darcs.Repository ( Repository, withRepository, RepoJob(..), repoCache )
 -- get at the caches and inspect them directly)
 -- Could move the relevant code into Darcs.Repository modules
 -- but it doesn't really seem worth it.
-import Darcs.Repository.Cache
+import Darcs.Util.Cache
     ( allHashedDirs
     , cacheEntries
     , hashedFilePath
@@ -58,7 +58,7 @@ import Darcs.UI.Commands
     , putInfo, putVerbose
     )
 import Darcs.UI.Completion ( noArgs )
-import Darcs.UI.Options ( (^), oid, odesc, ocheck, defaultFlags, (?) )
+import Darcs.UI.Options ( (^), oid, (?) )
 import qualified Darcs.UI.Options.All as O
 import Darcs.Util.Path ( AbsolutePath )
 import Darcs.UI.Flags ( DarcsFlag, useCache )
@@ -122,10 +122,7 @@ gzcrcs = DarcsCommand
     , commandPrereq = amInRepository
     , commandCompleteArgs = noArgs
     , commandArgdefaults = nodefaults
-    , commandAdvancedOptions = []
-    , commandBasicOptions = odesc gzcrcsBasicOpts
-    , commandDefaults = defaultFlags gzcrcsOpts
-    , commandCheckOptions = ocheck gzcrcsOpts
+    , commandOptions = gzcrcsOpts
     }
   where
     gzcrcsBasicOpts = O.gzcrcsActions ^ O.justThisRepo ^ O.repoDir
@@ -137,7 +134,7 @@ gzcrcsCmd _ opts _ =
     Nothing -> fail "You must specify --check or --repair for gzcrcs"
     Just action -> withRepository (useCache ? opts) (RepoJob (gzcrcs' action opts))
 
-gzcrcs' :: O.GzcrcsAction -> [DarcsFlag] -> Repository rt p wR wU wT -> IO ()
+gzcrcs' :: O.GzcrcsAction -> [DarcsFlag] -> Repository rt p wU wR -> IO ()
 gzcrcs' action opts repo = do
     -- Somewhat ugly IORef use here because it's convenient, would be nicer to
     -- pre-filter the list of locs to check and then decide whether to print
