@@ -27,10 +27,11 @@ import Darcs.Prelude
 
 import Data.List ( intersect, find )
 import Data.List.Ordered ( nubSort )
-import Data.Maybe ( fromMaybe, isJust )
+import Data.Maybe ( catMaybes, fromMaybe, isJust )
 import Control.Arrow ( second )
 import Control.Exception ( catch, IOException )
-import Control.Monad.State.Strict
+import Control.Monad ( when, unless )
+import Control.Monad.State.Strict ( evalState, get, gets, modify )
 
 import Darcs.UI.PrintPatch ( showFriendly )
 import Darcs.Patch.PatchInfoAnd ( PatchInfoAndG, fmapFLPIAP, hopefullyM, info )
@@ -40,11 +41,11 @@ import Darcs.UI.Completion ( knownFileArgs )
 import Darcs.UI.External ( viewDocWith )
 import Darcs.UI.Flags
     ( DarcsFlag
-    , changesReverse, onlyToFiles
+    , changesReverse, onlyToFiles, diffingOpts
     , useCache, maxCount, hasXmlOutput
-    , verbosity, withContext, isInteractive, verbose
+    , verbosity, isInteractive, verbose
     , getRepourl, pathSetFromArgs )
-import Darcs.UI.Options ( (^), odesc, ocheck, defaultFlags, parseFlags, (?) )
+import Darcs.UI.Options ( (^), parseFlags, (?) )
 import qualified Darcs.UI.Options.All as O
 import Darcs.Util.Path
     ( SubPath
@@ -56,9 +57,8 @@ import Darcs.Util.Path
     )
 import Darcs.Repository ( PatchInfoAnd,
                           withRepositoryLocation, RepoJob(..),
-                          readRepo, unrecordedChanges,
+                          readPatches, unrecordedChanges,
                           withRepoLockCanFail )
-import Darcs.Repository.Flags ( UseIndex(..), ScanKnown(..), DiffAlgorithm(MyersDiff) )
 import Darcs.Util.Lock ( withTempDir )
 import Darcs.Patch.Set ( PatchSet, patchSet2RL, Origin )
 import Darcs.Patch.Format ( PatchListFormat )
@@ -151,10 +151,7 @@ log = DarcsCommand
     , commandCommand = logCmd
     , commandPrereq = findRepository
     , commandArgdefaults = nodefaults
-    , commandAdvancedOptions = odesc logAdvancedOpts
-    , commandBasicOptions = odesc logBasicOpts
-    , commandDefaults = defaultFlags logOpts
-    , commandCheckOptions = ocheck logOpts
+    , commandOptions = logOpts
     }
   where
     logBasicOpts
@@ -167,7 +164,7 @@ log = DarcsCommand
       ^ O.possiblyRemoteRepo
       ^ O.repoDir
       ^ O.interactive
-    logAdvancedOpts = O.network ^ O.patchIndexYes
+    logAdvancedOpts = O.remoteDarcs ^ O.patchIndexYes
     logOpts = logBasicOpts `withStdOpts` logAdvancedOpts
 
 logCmd :: (AbsolutePath, AbsolutePath) -> [DarcsFlag] -> [String] -> IO ()
@@ -179,7 +176,7 @@ logCmd fps opts args
       (fs, es) <- remoteSubPaths args []
       if null es then
         withTempDir "darcs.log"
-          (\_ -> showLog opts $ maybeNotNull $ nubSort $ map floatSubPath fs)
+          (\_ -> showLog opts $ maybeNotNull $ nubSort $ filterErrors $ map floatSubPath fs)
       else
         fail $ "For a remote repo I can only handle relative paths.\n"
             ++ "Invalid arguments: "++unwords es
@@ -188,7 +185,7 @@ logCmd fps opts args
       unless (isInteractive False opts)
         $ when (O.patchIndexNo ? opts == O.YesPatchIndex)
           $ withRepoLockCanFail (useCache ? opts)
-            $ RepoJob (\repo -> readRepo repo >>= attemptCreatePatchIndex repo)
+            $ RepoJob (\repo -> readPatches repo >>= attemptCreatePatchIndex repo)
       paths <- pathSetFromArgs fps args
       showLog opts paths
 
@@ -196,6 +193,9 @@ maybeNotNull :: [a] -> Maybe [a]
 maybeNotNull [] = Nothing
 maybeNotNull xs = Just xs
 
+filterErrors :: [Either e a] -> [a]
+filterErrors = catMaybes . map (either (const Nothing) Just)
+
 hasRemoteRepo :: [DarcsFlag] -> Bool
 hasRemoteRepo = isJust . getRepourl
 
@@ -214,12 +214,12 @@ showLog opts files =
   unless (O.debug ? opts) $ setProgressMode False
   Sealed unrec <- case files of
     Nothing -> return $ Sealed NilFL
-    Just _ -> Sealed `fmap` unrecordedChanges (UseIndex, ScanKnown, MyersDiff)
-                  O.NoLookForMoves O.NoLookForReplaces
-                  repository files
-                  `catch` \(_ :: IOException) -> return (Sealed NilFL) -- this is triggered when repository is remote
+    Just _ ->
+      Sealed `fmap` unrecordedChanges (diffingOpts opts) repository files `catch`
+        \(_ :: IOException) ->
+          return (Sealed NilFL) -- this is triggered when repository is remote
   debugMessage "About to read the repository..."
-  patches <- readRepo repository
+  patches <- readPatches repository
   debugMessage "Done reading the repository."
   let recFiles = effectOnPaths (invert unrec) <$> files
       filtered_changes p =
@@ -242,7 +242,7 @@ showLog opts files =
                     _ -> mempty
             debugMessage "About to print the patches..."
             let printers = if hasXmlOutput opts then simplePrinters else fancyPrinters
-            ps <- readRepo repository -- read repo again to prevent holding onto
+            ps <- readPatches repository -- read repo again to prevent holding onto
                                        -- values forced by filtered_changes
             logOutput <- changelog opts (patchSet2RL ps) `fmap` filtered_changes patches
             viewDocWith printers (header $$ logOutput)
@@ -272,21 +272,21 @@ simpleLogInfo :: ( MatchableRP p
                  , ApplyState p ~ Tree
                  )
               => AnchoredPath
-              -> PatchFilter rt p
-              -> PatchSet rt p Origin wY
-              -> IO [Sealed2 (PatchInfoAnd rt p)]
+              -> PatchFilter p
+              -> PatchSet p Origin wY
+              -> IO [Sealed2 (PatchInfoAnd p)]
 simpleLogInfo path pf ps =
   map fst . liPatches <$> getLogInfo Nothing [] False (Just [path]) pf ps
 
-getLogInfo :: forall rt p wY.
+getLogInfo :: forall p wY.
               ( MatchableRP p
               , ApplyState p ~ Tree
               )
            => Maybe Int -> [MatchFlag] -> Bool
            -> Maybe [AnchoredPath]
-           -> PatchFilter rt p
-           -> PatchSet rt p Origin wY
-           -> IO (LogInfo (PatchInfoAnd rt p))
+           -> PatchFilter p
+           -> PatchSet p Origin wY
+           -> IO (LogInfo (PatchInfoAnd p))
 getLogInfo maxCountFlag matchFlags onlyToFilesFlag paths patchFilter ps =
   case matchRange matchFlags ps of
     Sealed2 range ->
@@ -322,14 +322,14 @@ getLogInfo maxCountFlag matchFlags onlyToFilesFlag paths patchFilter ps =
 -- "Just n" -- returns at most n patches touching the file (starting from the
 -- beginning of the patch list).
 filterPatchesByNames
-    :: forall rt p.
+    :: forall p.
        ( MatchableRP p
        , ApplyState p ~ Tree
        )
     => Maybe Int                      -- ^ maxcount
     -> [AnchoredPath]                 -- ^ paths
-    -> [Sealed2 (PatchInfoAnd rt p)]  -- ^ patches
-    -> LogInfo (PatchInfoAnd rt p)
+    -> [Sealed2 (PatchInfoAnd p)]  -- ^ patches
+    -> LogInfo (PatchInfoAnd p)
 filterPatchesByNames maxcount paths patches = removeNonRenames $
     evalState (filterPatchesByNamesM paths patches) (maxcount, initRenames) where
         removeNonRenames li = li { liRenames = removeIds (liRenames li) }
@@ -363,12 +363,12 @@ filterPatchesByNames maxcount paths patches = removeNonRenames $
                                 modify $ second (const renames')
                                 filterPatchesByNamesM fs' ps
 
-changelog :: forall rt p wStart wX
+changelog :: forall p wStart wX
            . ( ShowPatch p, PatchListFormat p
              , Summary p, HasDeps p, PrimDetails (PrimOf p)
              )
-          => [DarcsFlag] -> RL (PatchInfoAndG rt p) wStart wX
-          -> LogInfo (PatchInfoAndG rt p)
+          => [DarcsFlag] -> RL (PatchInfoAndG p) wStart wX
+          -> LogInfo (PatchInfoAndG p)
           -> Doc
 changelog opts patches li
     | O.changesFormat ? opts == Just O.CountPatches =
@@ -379,7 +379,7 @@ changelog opts patches li
     | otherwise = vsep (map (number_patch description') ps) $$ mbErr
     where ps_and_fs = liPatches li
           mbErr = fromMaybe mempty (liErrorMsg li)
-          change_with_summary :: Sealed2 (PatchInfoAndG rt p) -> Doc
+          change_with_summary :: Sealed2 (PatchInfoAndG p) -> Doc
           change_with_summary (Sealed2 hp)
             | Just p <- hopefullyM hp =
               if O.changesFormat ? opts == Just O.MachineReadable
@@ -394,7 +394,7 @@ changelog opts patches li
             , text "</changelog>"
             ]
 
-          xml_with_summary :: Sealed2 (PatchInfoAndG rt p) -> Doc
+          xml_with_summary :: Sealed2 (PatchInfoAndG p) -> Doc
           xml_with_summary (Sealed2 hp) | Just p <- hopefullyM hp =
                     let
                       deps = getdeps p
@@ -434,10 +434,10 @@ changelog opts patches li
                                   Nothing -> f x
                              else f x
 
-          get_number :: Sealed2 (PatchInfoAndG re p) -> Maybe Int
+          get_number :: Sealed2 (PatchInfoAndG p) -> Maybe Int
           get_number (Sealed2 y) = gn 1 patches
               where iy = info y
-                    gn :: Int -> RL (PatchInfoAndG rt p) wStart wY -> Maybe Int
+                    gn :: Int -> RL (PatchInfoAndG p) wStart wY -> Maybe Int
                     gn n (bs:<:b) | seq n (info b) == iy = Just n
                                   | otherwise = gn (n+1) bs
                     gn _ NilRL = Nothing
@@ -448,7 +448,7 @@ logContext :: [DarcsFlag] -> IO ()
 logContext opts = do
   let repodir = fromMaybe "." $ getRepourl opts
   withRepositoryLocation (useCache ? opts) repodir $ RepoJob $ \repository -> do
-      (_ :> ps) <- contextPatches `fmap` readRepo repository
+      (_ :> ps) <- contextPatches `fmap` readPatches repository
       let header = text "\nContext:\n"
       viewDocWith simplePrinters $ vsep
           (header : mapRL (showPatchInfo ForStorage . info) ps)
@@ -474,5 +474,4 @@ logPatchSelOpts flags = S.PatchSelectionOptions
     , S.interactive = isInteractive False flags
     , S.selectDeps = O.PromptDeps -- option not supported, use default
     , S.withSummary = O.withSummary ? flags
-    , S.withContext = withContext ? flags
     }
