@@ -20,48 +20,42 @@ module Darcs.UI.Commands.Repair ( repair, check ) where
 
 import Darcs.Prelude
 
-import Control.Monad ( when, unless )
-import Control.Exception ( catch, IOException )
-import System.Exit ( ExitCode(..), exitWith )
+import Control.Monad ( when, unless, void )
+import Data.Maybe ( isJust )
+import System.IO.Error ( catchIOError )
+import System.Exit ( exitFailure )
 import System.Directory( renameFile )
 import System.FilePath ( (<.>) )
 
 import Darcs.UI.Commands
     ( DarcsCommand(..), withStdOpts, nodefaults
-    , putInfo, putWarning, amInHashedRepository
+    , putInfo, putVerbose, putWarning, amInHashedRepository
     )
 import Darcs.UI.Completion ( noArgs )
 import Darcs.UI.Flags
     ( DarcsFlag, verbosity, umask, useIndex
-    , useCache, compress, diffAlgorithm, quiet
-    )
-import Darcs.UI.Options
-    ( DarcsOption, (^), oid
-    , odesc, ocheck, defaultFlags, (?)
+    , useCache, diffAlgorithm, quiet
     )
+import Darcs.UI.Options ( DarcsOption, oid, (?), (^) )
 import qualified Darcs.UI.Options.All as O
 
-import Darcs.Repository.Flags ( UpdatePending (..) )
 import Darcs.Repository.Paths ( indexPath )
 import Darcs.Repository.Repair
     ( replayRepository, checkIndex, replayRepositoryInTemp
     , RepositoryConsistency(..)
     )
 import Darcs.Repository
-    ( Repository, withRepository, readRecorded, RepoJob(..)
-    , withRepoLock, replacePristine, repoCache
+    ( withRepository, RepoJob(..)
+    , withRepoLock, writePristine
+    , finalizeRepositoryChanges
     )
-import qualified Darcs.Repository.Hashed as HashedRepo
-import Darcs.Repository.Prefs ( filetypeFunction )
-import Darcs.Repository.Diff( treeDiff )
+import Darcs.Repository.Hashed ( writeTentativeInventory )
+import Darcs.Repository.Pending ( setTentativePending )
 
-import Darcs.Patch ( RepoPatch, PrimOf, displayPatch )
-import Darcs.Patch.Witnesses.Ordered ( FL(..) )
-import Darcs.Patch.Witnesses.Sealed ( Sealed(..), unFreeLeft )
+import Darcs.Patch ( displayPatch )
+import Darcs.Patch.Witnesses.Sealed ( Sealed(..) )
 
-import Darcs.Util.Printer ( Doc, text )
-import Darcs.Util.Tree ( Tree, expand )
-import Darcs.Util.Tree.Hashed ( darcsUpdateHashes )
+import Darcs.Util.Printer ( Doc, text, ($$) )
 
 
 repairDescription :: String
@@ -82,8 +76,8 @@ repairHelp = text $
   \are enountered.\n"
 
 commonBasicOpts :: DarcsOption a
-                   (Maybe String -> O.UseIndex -> O.DiffAlgorithm -> a)
-commonBasicOpts = O.repoDir ^ O.useIndex ^ O.diffAlgorithm
+                   (Maybe String -> O.DiffAlgorithm -> a)
+commonBasicOpts = O.repoDir ^ O.diffAlgorithm
 
 repair :: DarcsCommand
 repair = DarcsCommand
@@ -103,41 +97,43 @@ repair = DarcsCommand
     basicOpts = commonBasicOpts ^ O.dryRun
     advancedOpts = O.umask
     allOpts = basicOpts `withStdOpts` advancedOpts
-    commandAdvancedOptions = odesc advancedOpts
-    commandBasicOptions = odesc basicOpts
-    commandDefaults = defaultFlags allOpts
-    commandCheckOptions = ocheck allOpts
+    commandOptions = allOpts
 
 withFpsAndArgs :: (b -> d) -> a -> b -> c -> d
 withFpsAndArgs cmd _ opts _ = cmd opts
 
+maybeDo :: Monad m => Maybe t -> (t -> m ()) -> m ()
+maybeDo (Just x) f = f x
+maybeDo Nothing _ = return ()
+
 repairCmd :: [DarcsFlag] -> IO ()
 repairCmd opts
   | O.yes (O.dryRun ? opts) = checkCmd opts
   | otherwise =
-    withRepoLock O.NoDryRun (useCache ? opts) YesUpdatePending (umask ? opts) $
+    withRepoLock (useCache ? opts) (umask ? opts) $
     RepoJob $ \repo -> do
-      replayRepository
+      bad_replay <- replayRepository
         (diffAlgorithm ? opts)
         repo
-        (compress ? opts)
-        (verbosity ? opts) $ \state ->
-        case state of
-          RepositoryConsistent ->
-            putInfo opts "The repository is already consistent, no changes made."
-          BrokenPristine tree -> do
-            putInfo opts "Fixing pristine tree..."
-            replacePristine repo tree
-          BrokenPatches tree newps -> do
+        (verbosity ? opts) $ \RepositoryConsistency {..} -> do
+          maybeDo fixedPatches $ \ps -> do
             putInfo opts "Writing out repaired patches..."
-            HashedRepo.writeTentativeInventory (repoCache repo) (compress ? opts) newps
-            HashedRepo.finalizeTentativeChanges repo (compress ? opts)
+            writeTentativeInventory repo ps
+          maybeDo fixedPristine $ \(tree, Sealed diff) -> do
+            putVerbose opts $ "Pristine differences:" $$ displayPatch diff
             putInfo opts "Fixing pristine tree..."
-            replacePristine repo tree
+            void $ writePristine repo tree
+          maybeDo fixedPending $ \(Sealed pend) -> do
+            putInfo opts "Writing out repaired pending..."
+            setTentativePending repo pend
+          return $ isJust fixedPatches || isJust fixedPristine || isJust fixedPending
       index_ok <- checkIndex repo (quiet opts)
       unless index_ok $ do
         renameFile indexPath (indexPath <.> "bad")
         putInfo opts "Bad index discarded."
+      if bad_replay || not index_ok
+        then void $ finalizeRepositoryChanges repo (O.dryRun ? opts)
+        else putInfo opts "The repository is already consistent, no changes made."
 
 -- |check is an alias for repair, with implicit DryRun flag.
 check :: DarcsCommand
@@ -157,48 +153,29 @@ check = DarcsCommand
     basicOpts = commonBasicOpts
     advancedOpts = oid
     allOpts = basicOpts `withStdOpts` advancedOpts
-    commandAdvancedOptions = odesc advancedOpts
-    commandBasicOptions = odesc basicOpts
-    commandDefaults = defaultFlags allOpts
-    commandCheckOptions = ocheck allOpts
+    commandOptions = allOpts
     commandDescription = "Alias for `darcs " ++ commandName repair ++ " --dry-run'."
 
 checkCmd :: [DarcsFlag] -> IO ()
 checkCmd opts = withRepository (useCache ? opts) $ RepoJob $ \repository -> do
-  state <- replayRepositoryInTemp (diffAlgorithm ? opts) repository (compress ? opts) (verbosity ? opts)
-  failed <-
-    case state of
-      RepositoryConsistent -> do
-        putInfo opts "The repository is consistent!"
-        return False
-      BrokenPristine newpris -> do
-        brokenPristine opts repository newpris
-        return True
-      BrokenPatches newpris _ -> do
-        brokenPristine opts repository newpris
-        putInfo opts "Found broken patches."
-        return True
-  bad_index <- if useIndex ? opts == O.IgnoreIndex
-                 then return False
-                 else not <$> checkIndex repository (quiet opts)
+  RepositoryConsistency {..} <-
+    replayRepositoryInTemp (diffAlgorithm ? opts) repository (verbosity ? opts)
+  maybeDo fixedPatches $ \_ ->
+    putInfo opts "Found broken patches."
+  maybeDo fixedPristine $ \(_, Sealed diff) -> do
+    putInfo opts "Found broken pristine tree."
+    putVerbose opts $ "Differences:" $$ displayPatch diff
+  maybeDo fixedPending $ \_ ->
+    putInfo opts "Found broken pending."
+  bad_index <-
+    if useIndex ? opts == O.IgnoreIndex
+      then return False
+      else
+        not <$> do
+          checkIndex repository (quiet opts) `catchIOError` \e -> do
+            putWarning opts ("Warning, cannot access the index:" $$ text (show e))
+            return True
   when bad_index $ putInfo opts "Bad index."
-  exitWith $ if failed || bad_index then ExitFailure 1 else ExitSuccess
-
-brokenPristine
-  :: forall rt p wR wU wT . (RepoPatch p)
-  => [DarcsFlag] -> Repository rt p wR wU wT -> Tree IO -> IO ()
-brokenPristine opts repository newpris = do
-  putInfo opts "Looks like we have a difference..."
-  mc' <-
-    (Just `fmap` (readRecorded repository >>= expand >>= darcsUpdateHashes))
-      `catch` (\(_ :: IOException) -> return Nothing)
-  case mc' of
-    Nothing -> do
-      putWarning opts $ "Unable to read the recorded state, try repair."
-    Just mc -> do
-      ftf <- filetypeFunction
-      Sealed (diff :: FL (PrimOf p) wR wR2)
-        <- unFreeLeft `fmap` treeDiff (diffAlgorithm ? opts) ftf newpris mc :: IO (Sealed (FL (PrimOf p) wR))
-      putInfo opts $ case diff of
-        NilFL -> "Nothing"
-        patch -> displayPatch patch
+  if isJust fixedPatches || isJust fixedPristine || isJust fixedPending || bad_index
+    then exitFailure
+    else putInfo opts "The repository is consistent!"
