@@ -7,7 +7,7 @@ import Darcs.UI.Commands
 import Darcs.UI.Completion ( noArgs )
 import Darcs.UI.Flags ( DarcsFlag, useCache, verbose )
 import Darcs.UI.Options
-    ( (^), oid, odesc, ocheck, defaultFlags, (?) )
+    ( (^), oid, (?) )
 import qualified Darcs.UI.Options.All as O
 import Darcs.Util.Path ( AbsolutePath )
 import Darcs.Repository ( withRepository, RepoJob(..), repoLocation )
@@ -32,10 +32,7 @@ showPatchIndex = DarcsCommand
     , commandPrereq = amInHashedRepository
     , commandCompleteArgs = noArgs
     , commandArgdefaults = nodefaults
-    , commandAdvancedOptions = []
-    , commandBasicOptions = odesc showPatchIndexBasicOpts
-    , commandDefaults = defaultFlags showPatchIndexOpts
-    , commandCheckOptions = ocheck showPatchIndexOpts
+    , commandOptions = showPatchIndexOpts
     }
   where
     showPatchIndexBasicOpts = O.nullFlag ^ O.repoDir
