@@ -16,6 +16,7 @@
 --  Boston, MA 02110-1301, USA.
 
 {-# LANGUAGE OverloadedStrings #-}
+{-# OPTIONS_GHC -Wno-incomplete-record-updates #-}
 module Darcs.UI.Commands.WhatsNew
     ( whatsnew
     , status
@@ -26,11 +27,12 @@ import Darcs.Prelude
 import Control.Monad ( void, when )
 import Control.Monad.Reader ( runReaderT )
 import Control.Monad.State ( evalStateT, liftIO )
+import Data.Maybe ( isJust )
 import System.Exit ( ExitCode (..), exitSuccess, exitWith )
 
 import Darcs.Patch
     ( PrimOf, PrimPatch, RepoPatch
-    , applyToTree, plainSummaryPrims, primIsHunk
+    , applyToTree, plainSummaryPrims
     )
 import Darcs.Patch.Apply ( ApplyState )
 import Darcs.Patch.Choices ( mkPatchChoices, labelPatches, unLabel )
@@ -55,8 +57,8 @@ import Darcs.Patch.Witnesses.Sealed
     , unFreeLeft
     )
 import Darcs.Repository
-    ( RepoJob (..), Repository
-    , readRecorded
+    ( RepoJob (..), Repository, AccessType(RO)
+    , readPristine
     , unrecordedChanges, withRepository
     )
 import Darcs.Repository.Diff ( treeDiff )
@@ -70,15 +72,13 @@ import Darcs.UI.Commands.Util ( announceFiles, filterExistingPaths )
 import Darcs.UI.External ( viewDocWith )
 import Darcs.UI.Flags
     ( DarcsFlag, diffAlgorithm
-    , withContext, useCache, pathSetFromArgs
+    , useCache, pathSetFromArgs
     , verbosity, isInteractive
-    , lookForAdds, lookForMoves, lookForReplaces
-    , scanKnown, useIndex, diffingOpts
+    , diffingOpts
     )
-import Darcs.UI.Options
-    ( DarcsOption, (^), odesc, ocheck, defaultFlags, parseFlags, (?) )
+import Darcs.UI.Options ( (^), parseFlags, (?), oid )
 import qualified Darcs.UI.Options.All as O
-import Darcs.UI.PrintPatch ( contextualPrintPatch )
+import Darcs.UI.PrintPatch ( contextualPrintPatchWithPager )
 import Darcs.UI.SelectChanges
     ( InteractiveSelectionM, KeyPress (..)
     , WhichChanges (..)
@@ -94,16 +94,13 @@ import Darcs.UI.SelectChanges
 import qualified Darcs.UI.SelectChanges as S ( PatchSelectionOptions (..) )
 import Darcs.Util.Path ( AbsolutePath, AnchoredPath )
 import Darcs.Util.Printer
-    ( Doc, formatWords, putDocLn, renderString
+    ( Doc, formatWords, putDocLn, putDocLnWith, renderString
     , text, vcat, ($+$)
     )
 import Darcs.Util.Printer.Color ( fancyPrinters )
 import Darcs.Util.Prompt ( PromptConfig (..), promptChar )
 import Darcs.Util.Tree ( Tree )
 
-commonAdvancedOpts :: DarcsOption a (O.UseIndex -> O.IncludeBoring -> a)
-commonAdvancedOpts = O.useIndex ^ O.includeBoring
-
 patchSelOpts :: [DarcsFlag] -> S.PatchSelectionOptions
 patchSelOpts flags = S.PatchSelectionOptions
     { S.verbosity = verbosity ? flags
@@ -111,7 +108,6 @@ patchSelOpts flags = S.PatchSelectionOptions
     , S.interactive = isInteractive True flags
     , S.selectDeps = O.PromptDeps -- option not supported, use default
     , S.withSummary = getSummary flags
-    , S.withContext = withContext ? flags
     }
 
 -- lookForAdds and machineReadable set YesSummary
@@ -122,7 +118,7 @@ getSummary flags = case O.maybeSummary Nothing ? flags of
   Just O.NoSummary -> O.NoSummary
   Just O.YesSummary -> O.YesSummary
   Nothing
-    | O.yes (lookForAdds flags) -> O.YesSummary
+    | O.yes (O.lookforadds ? flags) -> O.YesSummary
     | O.machineReadable ? flags -> O.YesSummary
     | otherwise -> O.NoSummary
 
@@ -138,21 +134,20 @@ whatsnew = DarcsCommand
     , commandPrereq = amInRepository
     , commandCompleteArgs = modifiedFileArgs
     , commandArgdefaults = nodefaults
-    , commandAdvancedOptions = odesc commonAdvancedOpts
-    , commandBasicOptions = odesc whatsnewBasicOpts
-    , commandDefaults = defaultFlags whatsnewOpts
-    , commandCheckOptions = ocheck whatsnewOpts
+    , commandOptions = whatsnewOpts
     }
   where
     whatsnewBasicOpts
       = O.maybeSummary Nothing
       ^ O.withContext
       ^ O.machineReadable
-      ^ O.lookfor
+      ^ O.maybelookforadds O.NoLookForAdds
+      ^ O.lookforreplaces
+      ^ O.lookformoves
       ^ O.diffAlgorithm
       ^ O.repoDir
       ^ O.interactive -- False
-    whatsnewOpts = whatsnewBasicOpts `withStdOpts` commonAdvancedOpts
+    whatsnewOpts = withStdOpts whatsnewBasicOpts oid
 
 whatsnewDescription :: String
 whatsnewDescription = "List unrecorded changes in the working tree."
@@ -191,8 +186,10 @@ whatsnewHelp =
   $+$ formatWords
   [ "By default, `darcs whatsnew` uses Darcs' internal format for changes."
   , "To see some context (unchanged lines) around each change, use the"
-  , "`--unified` option.  To view changes in conventional `diff` format, use"
-  , "the `darcs diff` command; but note that `darcs whatsnew` is faster."
+  , "`--unified` option. (This option has no effect in interactive mode.)"
+  , "To view changes in conventional `diff` format, use"
+  , "the `darcs diff` command; but note that `darcs diff` cannot properly"
+  , "display changes when file renames are involved."
   ]
   $+$ formatWords
   [ "This command exits unsuccessfully (returns a non-zero exit status) if"
@@ -201,14 +198,12 @@ whatsnewHelp =
 
 whatsnewCmd :: (AbsolutePath, AbsolutePath) -> [DarcsFlag] -> [String] -> IO ()
 whatsnewCmd fps opts args =
-   withRepository (useCache ? opts) $ RepoJob $ \(repo :: Repository rt p wR wU wR) -> do
-    let scan = scanKnown (lookForAdds opts) (O.includeBoring ? opts)
+   withRepository (useCache ? opts) $ RepoJob $ \(repo :: Repository 'RO p wU wR) -> do
     existing_files <- do
       files <- pathSetFromArgs fps args
-      files' <- traverse
-        (filterExistingPaths
-          repo (verbosity ? opts) (useIndex ? opts) scan (lookForMoves opts))
-        files
+      files' <-
+        traverse
+          (filterExistingPaths repo (verbosity ? opts) (diffingOpts opts)) files
       let files'' = fmap snd files'
       when (files'' == Just []) $
         fail "None of the files you specified exist."
@@ -217,12 +212,10 @@ whatsnewCmd fps opts args =
     -- get all unrecorded changes, possibly including unadded or even boring
     -- files if the appropriate options were supplied
     Sealed allInterestingChanges <-
-      filteredUnrecordedChanges (diffingOpts opts)
-        (lookForMoves opts) (lookForReplaces opts)
-        repo existing_files
+      filteredUnrecordedChanges (diffingOpts opts) repo existing_files
 
     -- get the recorded state
-    pristine <- readRecorded repo
+    pristine <- readPristine repo
 
     -- the case --look-for-adds and --summary must be handled specially
     -- in order to distinguish added and unadded files
@@ -235,9 +228,9 @@ whatsnewCmd fps opts args =
       if haveLookForAddsAndSummary
         then
           -- do *not* look for adds here:
-          filteredUnrecordedChanges (O.useIndex ? opts, O.ScanKnown, O.diffAlgorithm ? opts)
-            (lookForMoves opts) (lookForReplaces opts)
-            repo existing_files
+          let dopts = diffingOpts opts
+          in filteredUnrecordedChanges
+                dopts {O.lookForAdds = O.NoLookForAdds} repo existing_files
         else return (Sealed NilFL)
     Sealed unaddedNewPathsPs <-
       if haveLookForAddsAndSummary
@@ -255,8 +248,7 @@ whatsnewCmd fps opts args =
     announceFiles (verbosity ? opts) existing_files "What's new in"
     if maybeIsInteractive opts
       then
-        runInteractive (interactiveHunks pristine) (patchSelOpts opts)
-          pristine allInterestingChanges
+        runInteractive interactiveHunks (patchSelOpts opts) allInterestingChanges
       else
         if haveLookForAddsAndSummary
           then do
@@ -266,12 +258,12 @@ whatsnewCmd fps opts args =
             printChanges pristine allInterestingChanges
   where
     haveSummary = O.yes (getSummary opts)
-    haveLookForAddsAndSummary = haveSummary && O.yes (lookForAdds opts)
+    haveLookForAddsAndSummary = haveSummary && O.yes (O.lookforadds ? opts)
 
     -- Filter out hunk patches (leaving add patches) and return the tree
     -- resulting from applying the filtered patches to the pristine tree.
     applyAddPatchesToPristine ps pristine = do
-        adds :> _ <- return $ partitionRL primIsHunk $ reverseFL ps
+        adds :> _ <- return $ partitionRL (isJust . isHunk) $ reverseFL ps
         applyToTree (reverseRL adds) pristine
 
     exitOnNoChanges :: FL p wX wY -> IO ()
@@ -303,40 +295,37 @@ whatsnewCmd fps opts args =
                  -> IO ()
     printChanges pristine changes
         | haveSummary = putDocLn $ plainSummaryPrims machineReadable changes
-        | O.yes (withContext ? opts) = contextualPrintPatch pristine changes
+        | O.yes (O.withContext ? opts) = contextualPrintPatchWithPager pristine changes
         | otherwise = printPatchPager changes
      where machineReadable = parseFlags O.machineReadable opts
 
     -- return the unrecorded changes that affect an optional list of paths.
     filteredUnrecordedChanges :: forall rt p wR wU. (RepoPatch p, ApplyState p ~ Tree)
-                              => (O.UseIndex, O.ScanKnown, O.DiffAlgorithm)
-                              -> O.LookForMoves
-                              -> O.LookForReplaces
-                              -> Repository rt p wR wU wR
+                              => O.DiffOpts
+                              -> Repository rt p wU wR
                               -> Maybe [AnchoredPath]
                               -> IO (Sealed (FL (PrimOf p) wR))
-    filteredUnrecordedChanges diffing lfm lfr repo paths =
-        chooseTouching paths <$> unrecordedChanges diffing lfm lfr repo paths
+    filteredUnrecordedChanges diffing repo paths =
+        chooseTouching paths <$> unrecordedChanges diffing repo paths
 
 -- | Runs the 'InteractiveSelectionM' code
 runInteractive :: InteractiveSelectionM p wX wY () -- Selection to run
                -> S.PatchSelectionOptions
-               -> Tree IO         -- Pristine
                -> FL p wX wY      -- A list of patches
                -> IO ()
-runInteractive i patchsel pristine ps' = do
+runInteractive i patchsel ps' = do
     let lps' = labelPatches Nothing ps'
         choices' = mkPatchChoices lps'
         ps = evalStateT i (initialSelectionState lps' choices')
     void $
       runReaderT ps $
-        selectionConfigPrim First "view" patchsel Nothing Nothing (Just pristine)
+        selectionConfigPrim First "view" patchsel Nothing Nothing
 
 -- | The interactive part of @darcs whatsnew@
 interactiveHunks :: (IsHunk p, ShowPatch p, ShowContextPatch p, Commute p,
                      PatchInspect p, PrimDetails p, ApplyState p ~ Tree)
-                 => Tree IO -> InteractiveSelectionM p wX wY ()
-interactiveHunks pristine = do
+                 => InteractiveSelectionM p wX wY ()
+interactiveHunks = do
     c <- currentPatch
     case c of
         Nothing -> liftIO $ putStrLn "No more changes!"
@@ -350,14 +339,14 @@ interactiveHunks pristine = do
                 (PromptConfig thePrompt (keysFor basic_options) (keysFor adv_options)
                  (Just 'n') "?h")
         case yorn of
-            -- View change in context
-            'v' -> liftIO (contextualPrintPatch pristine (unLabel lp))
+            -- View change
+            'v' -> liftIO (printPatch (unLabel lp))
                    >> repeatThis lp
             -- View summary of the change
             'x' -> liftIO (putDocLn $ summary $ unLabel lp)
                    >> repeatThis lp
             -- View change and move on
-            'y' -> liftIO (contextualPrintPatch pristine (unLabel lp))
+            'y' -> liftIO (printPatch (unLabel lp))
                    >> decide True lp >> next_hunk
             -- Go to the next patch
             'n' -> decide False lp >> next_hunk
@@ -381,15 +370,16 @@ interactiveHunks pristine = do
             _ -> do liftIO . putStrLn $
                         helpFor "whatsnew" basic_options adv_options
                     repeatThis lp
-    start_over = backAll >> interactiveHunks pristine
-    next_hunk  = skipOne >> skipMundane >> interactiveHunks pristine
-    prev_hunk  = backOne >> interactiveHunks pristine
+    start_over = backAll >> interactiveHunks
+    next_hunk  = skipOne >> skipMundane >> interactiveHunks
+    prev_hunk  = backOne >> interactiveHunks
     options_yn =
-        [ KeyPress 'v' "view this change in a context"
-        , KeyPress 'y' "view this change in a context and go to the next one"
-        , KeyPress 'n' "skip this change and its dependencies" ]
+        [ KeyPress 'v' "view this change"
+        , KeyPress 'y' "view this change and go to the next one"
+        , KeyPress 'n' "skip this change and its dependencies"
+        ]
     optionsView =
-        [ KeyPress 'p' "view this change in context wih pager "
+        [ KeyPress 'p' "view this change with pager"
         , KeyPress 'x' "view a summary of this change"
         ]
     optionsNav =
@@ -405,15 +395,15 @@ interactiveHunks pristine = do
 printPatchPager :: ShowPatchBasic p => p wX wY -> IO ()
 printPatchPager = viewDocWith fancyPrinters . displayPatch
 
+printPatch :: ShowPatchBasic p => p wX wY -> IO ()
+printPatch = putDocLnWith fancyPrinters . displayPatch
+
 -- | An alias for 'whatsnew', with implicit @-l@ (and thus implicit @-s@)
 -- flags. We override the default description, to include these flags.
 status :: DarcsCommand
 status = statusAlias
     { commandDescription = statusDesc
-    , commandAdvancedOptions = odesc commonAdvancedOpts
-    , commandBasicOptions = odesc statusBasicOpts
-    , commandDefaults = defaultFlags statusOpts
-    , commandCheckOptions = ocheck statusOpts
+    , commandOptions = statusOpts
     }
   where
     statusAlias = commandAlias "status" Nothing whatsnew
@@ -422,13 +412,13 @@ status = statusAlias
       = O.maybeSummary (Just O.YesSummary)
       ^ O.withContext
       ^ O.machineReadable
-      ^ O.lookforadds O.YesLookForAdds
+      ^ O.maybelookforadds O.YesLookForAdds
       ^ O.lookforreplaces
       ^ O.lookformoves
       ^ O.diffAlgorithm
       ^ O.repoDir
       ^ O.interactive
-    statusOpts = statusBasicOpts `withStdOpts` commonAdvancedOpts
+    statusOpts = withStdOpts statusBasicOpts oid
 
 maybeIsInteractive :: [DarcsFlag] -> Bool
 maybeIsInteractive = maybe False id . parseFlags O.interactive
