@@ -10,10 +10,10 @@ module Darcs.Util.Prompt
     ) where
 
 
-import Prelude ()
 import Darcs.Prelude
 
 import Control.Monad ( void )
+import Control.Monad.Trans ( liftIO )
 
 import Data.Char ( toUpper, toLower, isSpace )
 
@@ -27,7 +27,7 @@ askUser :: String    -- ^ The prompt to display
         -> IO String -- ^ The string the user entered.
 askUser prompt = withoutProgress $ runInputT defaultSettings $
                     getInputLine prompt
-                        >>= maybe (error "askUser: unexpected end of input") return
+                        >>= maybe (liftIO $ fail "askUser: unexpected end of input") return
 
 -- | Ask the user to press Enter
 askEnter :: String  -- ^ The prompt to display
@@ -45,7 +45,7 @@ askUserListItem prompt xs = withoutProgress $ runInputT defaultSettings $ do
   where
     loop = do
       answer <- getInputLine prompt
-                  >>= maybe (error "askUser: unexpected end of input") return
+                  >>= maybe (liftIO $ fail "askUser: unexpected end of input") return
       case maybeRead answer of
         Just n | n > 0 && n <= length xs -> return (xs !! (n-1))
         _ -> outputStrLn "Invalid response, try again!" >> loop
@@ -78,17 +78,17 @@ promptYorn p = (== 'y') `fmap` promptChar (PromptConfig p "yn" [] Nothing [])
 --   selected when the user presses the space bar) is shown as uppercase,
 --   hence users may want to enter it as uppercase.
 promptChar :: PromptConfig -> IO Char
-promptChar (PromptConfig p basic_chs adv_chs md help_chs) =
+promptChar (PromptConfig p basic_chs adv_chs def_ch help_chs) =
   withoutProgress $ runInputT defaultSettings loopChar
  where
  chs = basic_chs ++ adv_chs
  loopChar = do
     let chars = setDefault (basic_chs ++ (if null adv_chs then "" else "..."))
         prompt = p ++ " [" ++ chars ++ "]" ++ helpStr
-    a <- getInputChar prompt >>= maybe (error "promptChar: unexpected end of input") (return . toLower)
+    a <- getInputChar prompt >>= maybe (liftIO $ fail "promptChar: unexpected end of input") (return . toLower)
     case () of
      _ | a `elem` chs                   -> return a
-       | a == ' '                       -> maybe tryAgain return md
+       | a == ' '                       -> maybe tryAgain return def_ch
        | a `elem` help_chs              -> return a
        | otherwise                      -> tryAgain
  helpStr = case help_chs of
@@ -97,7 +97,7 @@ promptChar (PromptConfig p basic_chs adv_chs md help_chs) =
                  | otherwise       -> ", or " ++ (h:" for more options: ")
  tryAgain = do outputStrLn "Invalid response, try again!"
                loopChar
- setDefault s = case md of Nothing -> s
-                           Just d  -> map (setUpper d) s
+ setDefault s = case def_ch of Nothing -> s
+                               Just d  -> map (setUpper d) s
  setUpper d c = if d == c then toUpper c else c
 
