@@ -6,7 +6,7 @@
 -- | The abstract representation of a Tree and useful abstract utilities to
 -- handle those.
 module Darcs.Util.Tree
-    ( Tree, Blob(..), TreeItem(..), ItemType(..), Hash(..)
+    ( Tree, Blob(..), TreeItem(..), ItemType(..), Hash
     , makeTree, makeTreeWithHash, emptyTree, emptyBlob, makeBlob, makeBlobBS
 
     -- * Unfolding stubbed (lazy) Trees.
@@ -20,7 +20,8 @@ module Darcs.Util.Tree
     , items, list, listImmediate, treeHash
     , lookup, find, findFile, findTree, itemHash, itemType
     , zipCommonFiles, zipFiles, zipTrees, diffTrees
-    , explodePath, explodePaths
+    , explodePath, explodePaths, locate, isDir
+    , treeHas, treeHasDir, treeHasFile, treeHasAnycase
 
     -- * Files (Blobs).
     , readBlob
@@ -50,16 +51,17 @@ import qualified Data.Map as M
 import Data.Maybe( catMaybes, isNothing )
 import Data.Either( lefts, rights )
 import Data.List( union, sort )
-import Control.Monad( filterM )
+import qualified Data.List
+import Control.Monad( filterM, join )
 
 --------------------------------
 -- Tree, Blob and friends
 --
 
-data Blob m = Blob !(m BL.ByteString) !Hash
+data Blob m = Blob !(m BL.ByteString) !(Maybe Hash)
 data TreeItem m = File !(Blob m)
                 | SubTree !(Tree m)
-                | Stub !(m (Tree m)) !Hash
+                | Stub !(m (Tree m)) !(Maybe Hash)
 
 data ItemType = TreeType | BlobType deriving (Show, Eq, Ord)
 
@@ -81,13 +83,13 @@ data Tree m = Tree { items :: M.Map Name (TreeItem m)
                    -- | Get hash of a Tree. This is guaranteed to uniquely
                    -- identify the Tree (including any blob content), as far as
                    -- cryptographic hashes are concerned. Sha256 is recommended.
-                   , treeHash :: !Hash }
+                   , treeHash :: !(Maybe Hash) }
 
 listImmediate :: Tree m -> [(Name, TreeItem m)]
 listImmediate = M.toList . items
 
 -- | Get a hash of a TreeItem. May be Nothing.
-itemHash :: TreeItem m -> Hash
+itemHash :: TreeItem m -> Maybe Hash
 itemHash (File (Blob _ h)) = h
 itemHash (SubTree t) = treeHash t
 itemHash (Stub _ h) = h
@@ -99,24 +101,24 @@ itemType (Stub _ _) = TreeType
 
 emptyTree :: Tree m
 emptyTree = Tree { items = M.empty
-                 , treeHash = NoHash }
+                 , treeHash = Nothing }
 
 emptyBlob :: (Monad m) => Blob m
-emptyBlob = Blob (return BL.empty) NoHash
+emptyBlob = Blob (return BL.empty) Nothing
 
 makeBlob :: (Monad m) => BL.ByteString -> Blob m
-makeBlob str = Blob (return str) (sha256 str)
+makeBlob str = Blob (return str) (Just $ sha256 str)
 
 makeBlobBS :: (Monad m) => B.ByteString -> Blob m
-makeBlobBS s' = let s = BL.fromChunks [s'] in Blob (return s) (sha256 s)
+makeBlobBS s' = let s = BL.fromChunks [s'] in Blob (return s) (Just $ sha256 s)
 
 makeTree :: [(Name,TreeItem m)] -> Tree m
 makeTree l = Tree { items = M.fromList l
-                  , treeHash = NoHash }
+                  , treeHash = Nothing }
 
 makeTreeWithHash :: [(Name,TreeItem m)] -> Hash -> Tree m
 makeTreeWithHash l h = Tree { items = M.fromList l
-                            , treeHash = h }
+                            , treeHash = Just h }
 
 -----------------------------------
 -- Tree access and lookup
@@ -160,6 +162,45 @@ list t_ = paths t_ (AnchoredPath [])
                     concat [ paths subt (appendPath p subn)
                              | (subn, SubTree subt) <- listImmediate t ]
 
+-- | Like 'find' but monadic and thus able to expand 'Stub's on the way.
+locate :: Monad m => Tree m -> AnchoredPath -> m (Maybe (TreeItem m))
+locate tree (AnchoredPath names) = go names (SubTree tree)
+  where
+    go [] i = return (Just i)
+    go _ (File _) = return Nothing
+    go ns (Stub mkTree _) = mkTree >>= go ns . SubTree
+    go (n:ns) (SubTree t) =
+      case lookup t n of
+        Nothing -> return Nothing
+        Just i -> go ns i
+
+isDir :: TreeItem m -> Bool
+isDir (File _) = False
+isDir _ = True
+
+treeHasAnycase :: Monad m
+               => Tree m
+               -> AnchoredPath
+               -> m Bool
+treeHasAnycase tree (AnchoredPath names) = go names (SubTree tree)
+  where
+    go [] _ = return True
+    go ns (Stub mkTree _) = mkTree >>= go ns . SubTree
+    go _ (File _) = return False
+    go (n:ns) (SubTree t) =
+      case Data.List.find (eqAnycase n . fst) (listImmediate t) of
+        Nothing -> return False
+        Just (_,i) -> go ns i
+
+treeHas :: Monad m => Tree m -> AnchoredPath -> m Bool
+treeHas tree path = maybe False (const True) <$> locate tree path
+
+treeHasDir :: Monad m => Tree m -> AnchoredPath -> m Bool
+treeHasDir tree path = maybe False isDir <$> locate tree path
+
+treeHasFile :: Monad m => Tree m -> AnchoredPath -> m Bool
+treeHasFile tree path = maybe False (not . isDir) <$> locate tree path
+
 -- | Like 'explodePath' but for multiple paths.
 explodePaths :: Tree IO -> [AnchoredPath] -> [AnchoredPath]
 explodePaths tree paths = concatMap (explodePath tree) paths
@@ -171,16 +212,19 @@ explodePath :: Tree m -> AnchoredPath -> [AnchoredPath]
 explodePath tree path =
   path : maybe [] (map (catPaths path . fst) . list) (findTree tree path)
 
-expandUpdate :: (Monad m) => (AnchoredPath -> Tree m -> m (Tree m)) -> Tree m -> m (Tree m)
-expandUpdate update t_ = go (AnchoredPath []) t_
-    where go path t = do
-            let subtree (name, sub) = do tree <- go (path `appendPath` name) =<< unstub sub
-                                         return (name, SubTree tree)
-            expanded <- mapM subtree [ x | x@(_, item) <- listImmediate t, isSub item ]
-            let orig_map = M.filter (not . isSub) (items t)
-                expanded_map = M.fromList expanded
-                tree = t { items = M.union orig_map expanded_map }
-            update path tree
+expandUpdate
+  :: (Monad m) => (AnchoredPath -> Tree m -> m (Tree m)) -> Tree m -> m (Tree m)
+expandUpdate update = go (AnchoredPath [])
+  where
+    go path t = do
+      let subtree (name, sub) = do
+            tree <- go (path `appendPath` name) =<< unstub sub
+            return (name, SubTree tree)
+      expanded <- mapM subtree [ x | x@(_, item) <- listImmediate t, isDir item ]
+      let orig_map     = M.filter (not . isDir) (items t)
+          expanded_map = M.fromList expanded
+          tree         = t { items = M.union orig_map expanded_map }
+      update path tree
 
 -- | Expand a stubbed Tree into a one with no stubs in it. You might want to
 -- filter the tree before expanding to save IO. This is the basic
@@ -196,7 +240,7 @@ expandPath :: (Monad m) => Tree m -> AnchoredPath -> m (Tree m)
 expandPath t (AnchoredPath []) = return t
 expandPath t (AnchoredPath (n:rest)) =
   case lookup t n of
-    (Just item) | isSub item -> amend t n rest =<< unstub item
+    (Just item) | isDir item -> amend t n rest =<< unstub item
     _ -> return t -- fail $ "Descent error in expandPath: " ++ show path_
     where
           amend t' name rest' sub = do
@@ -209,7 +253,7 @@ expandPath t (AnchoredPath (n:rest)) =
 -- where there are problems. The first argument is the hashing function
 -- used to create the tree.
 checkExpand :: (TreeItem IO -> IO Hash) -> Tree IO
-            -> IO (Either [(AnchoredPath, Hash, Maybe Hash)] (Tree IO))
+            -> IO (Either [(AnchoredPath, Maybe Hash, Maybe Hash)] (Tree IO))
 checkExpand hashFunc t = go (AnchoredPath []) t
     where
       go path t_ = do
@@ -225,22 +269,22 @@ checkExpand hashFunc t = go (AnchoredPath []) t
                               Left problems -> Left problems
                               Right tree -> Right (name, SubTree tree)
             badBlob (_, f@(File (Blob _ h))) =
-              fmap (/= h) (hashFunc f `catch` (\(_ :: IOException) -> return NoHash))
+              fmap (/= h) (fmap Just (hashFunc f) `catch` (\(_ :: IOException) -> return Nothing))
             badBlob _ = return False
             render (name, f@(File (Blob _ h))) = do
               h' <- (Just <$> hashFunc f) `catch` \(_ :: IOException) -> return Nothing
               return (path `appendPath` name, h, h')
-            render (name, _) = return (path `appendPath` name, NoHash, Nothing)
-        subs <- mapM subtree [ x | x@(_, item) <- listImmediate t_, isSub item ]
+            render (name, _) = return (path `appendPath` name, Nothing, Nothing)
+        subs <- mapM subtree [ x | x@(_, item) <- listImmediate t_, isDir item ]
         badBlobs <- filterM badBlob (listImmediate t) >>= mapM render
         let problems = badBlobs ++ concat (lefts subs)
         if null problems
          then do
-           let orig_map = M.filter (not . isSub) (items t)
+           let orig_map = M.filter (not . isDir) (items t)
                expanded_map = M.fromList $ rights subs
                tree = t_ {items = orig_map `M.union` expanded_map}
            h' <- hashFunc (SubTree t_)
-           if h' `match` treeHash t_
+           if Just h' `match` treeHash t_
             then return $ Right tree
             else return $ Left [(path, treeHash t_, Just h')]
          else return $ Left problems
@@ -253,17 +297,15 @@ class (Monad m) => FilterTree a m where
     filter :: (AnchoredPath -> TreeItem m -> Bool) -> a m -> a m
 
 instance (Monad m) => FilterTree Tree m where
-    filter predicate t_ = filter' t_ (AnchoredPath [])
-        where filter' t path = t { items = M.mapMaybeWithKey (wibble path) $ items t }
+    filter predicate = filter' (AnchoredPath [])
+        where filter' path t = t { items = M.mapMaybeWithKey (wibble path) $ items t }
               wibble path name item =
                   let npath = path `appendPath` name in
                       if predicate npath item
                          then Just $ filterSub npath item
                          else Nothing
-              filterSub npath (SubTree t) = SubTree $ filter' t npath
-              filterSub npath (Stub stub h) =
-                  Stub (do x <- stub
-                           return $ filter' x npath) h
+              filterSub npath (SubTree t) = SubTree (filter' npath t)
+              filterSub npath (Stub stub _) = Stub (filter' npath <$> stub) Nothing
               filterSub _ x = x
 
 -- | Given two Trees, a @guide@ and a @tree@, produces a new Tree that is a
@@ -378,15 +420,13 @@ diffTrees left right =
 modifyTree :: (Monad m) => Tree m -> AnchoredPath -> Maybe (TreeItem m) -> Tree m
 modifyTree t_ p_ i_ = snd $ go t_ p_ i_
   where fix t unmod items' = (unmod, t { items = (countmap items':: Int) `seq` items'
-                                       , treeHash = if unmod then treeHash t else NoHash })
+                                       , treeHash = if unmod then treeHash t else Nothing })
 
         go t (AnchoredPath []) (Just (SubTree sub)) = (treeHash t `match` treeHash sub, sub)
 
         go t (AnchoredPath [n]) (Just item) = fix t unmod items'
             where !items' = M.insert n item (items t)
-                  !unmod = itemHash item `match` case lookup t n of
-                                             Nothing -> NoHash
-                                             Just i -> itemHash i
+                  !unmod = itemHash item `match` join (fmap itemHash (lookup t n))
 
         go t (AnchoredPath [n]) Nothing = fix t unmod items'
             where !items' = M.delete n (items t)
@@ -400,7 +440,7 @@ modifyTree t_ p_ i_ = snd $ go t_ p_ i_
                   !sub' = case lookup t n of
                     Just (SubTree s) -> let (mod', sub'') = subtree s in (mod', SubTree sub'')
                     Just (Stub s _) -> (False, Stub (do x <- s
-                                                        return $! snd $! subtree x) NoHash)
+                                                        return $! snd $! subtree x) Nothing)
                     Nothing -> (False, SubTree $! snd $! subtree emptyTree)
                     _ -> error $ "Modify tree at " ++ show path
 
@@ -417,7 +457,7 @@ countmap = M.foldr (\_ i -> i + 1) 0
 updateSubtrees :: (Tree m -> Tree m) -> Tree m -> Tree m
 updateSubtrees fun t =
     fun $ t { items = M.mapWithKey (curry $ snd . update) $ items t
-            , treeHash = NoHash }
+            , treeHash = Nothing }
   where update (k, SubTree s) = (k, SubTree $ updateSubtrees fun s)
         update (k, File f) = (k, File f)
         update (_, Stub _ _) = error "Stubs not supported in updateTreePostorder"
@@ -433,7 +473,7 @@ partiallyUpdateTree fun predi t' = go (AnchoredPath []) t'
   where go path t = do
           items' <- M.fromList <$> mapM (maybeupdate path) (listImmediate t)
           subtree <- fun . SubTree $ t { items = items'
-                                       , treeHash = NoHash }
+                                       , treeHash = Nothing }
           case subtree of
             SubTree t'' -> return t''
             _ -> error "function passed to partiallyUpdateTree changed SubTree to something else"
@@ -449,41 +489,40 @@ partiallyUpdateTree fun predi t' = go (AnchoredPath []) t'
 -- object, nor it is allowed for the overlay to add new objects to base.  This
 -- means that the overlay Tree should be a subset of the base Tree (although
 -- any extraneous items will be ignored by the implementation).
-overlay :: (Monad m) => Tree m -> Tree m -> Tree m
-overlay base over = Tree { items = M.fromList immediate
-                         , treeHash = NoHash }
-    where immediate = [ (n, get n) | (n, _) <- listImmediate base ]
-          get n = case (M.lookup n $ items base, M.lookup n $ items over) of
-                    (Just (File _), Just f@(File _)) -> f
-                    (Just (SubTree b), Just (SubTree o)) -> SubTree $ overlay b o
-                    (Just (Stub b _), Just (SubTree o)) -> Stub (flip overlay o `fmap` b) NoHash
-                    (Just (SubTree b), Just (Stub o _)) -> Stub (overlay b `fmap` o) NoHash
-                    (Just (Stub b _), Just (Stub o _)) -> Stub (do o' <- o
-                                                                   b' <- b
-                                                                   return $ overlay b' o') NoHash
-                    (Just x, _) -> x
-                    (_, _) -> error $ "Unexpected case in overlay at get " ++ show n ++ "."
+overlay :: Applicative m => Tree m -> Tree m -> Tree m
+overlay base over = Tree {items = M.fromList immediate, treeHash = Nothing}
+  where
+    immediate = [(n, get n) | (n, _) <- listImmediate base]
+    get n =
+      case (M.lookup n $ items base, M.lookup n $ items over) of
+        (Just (File _), Just f@(File _)) -> f
+        (Just (SubTree b), Just (SubTree o)) -> SubTree $ overlay b o
+        (Just (Stub b _), Just (SubTree o)) -> Stub (overlay <$> b <*> pure o) Nothing
+        (Just (SubTree b), Just (Stub o _)) -> Stub (overlay <$> pure b <*> o) Nothing
+        (Just (Stub b _), Just (Stub o _)) -> Stub (overlay <$> b <*> o) Nothing
+        (Just x, _) -> x
+        (_, _) -> error $ "Unexpected case in overlay at get " ++ show n ++ "."
 
+-- | Calculate and insert hashes for all 'TreeItem's contained in a 'Tree',
+-- including the argument 'Tree' itself. If necessary, this expands 'Stub's.
 addMissingHashes :: (Monad m) => (TreeItem m -> m Hash) -> Tree m -> m (Tree m)
-addMissingHashes make = updateTree update -- use partiallyUpdateTree here
-    where update (SubTree t) = make (SubTree t) >>= \x -> return $ SubTree (t { treeHash = x })
-          update (File blob@(Blob con NoHash)) =
-              do hash <- make $ File blob
-                 return $ File (Blob con hash)
-          update (Stub s NoHash) = update . SubTree =<< s
+addMissingHashes make = updateTree update
+    where update item@(SubTree t) = do
+              hash <- make item
+              return $ SubTree (t { treeHash = Just hash })
+          update item@(File (Blob con Nothing)) = do
+              hash <- make item
+              return $ File (Blob con (Just hash))
+          update (Stub s Nothing) = update . SubTree =<< s
           update x = return x
 
------- Private utilities shared among multiple functions. --------
+-- ---- Private utilities shared among multiple functions. --------
 
 unstub :: (Monad m) => TreeItem m -> m (Tree m)
 unstub (Stub s _) = s
 unstub (SubTree s) = return s
 unstub _ = return emptyTree
 
-isSub :: TreeItem m -> Bool
-isSub (File _) = False
-isSub _ = True
-
 -- Properties
 
 -- | Specification of 'explodePath'
