@@ -20,8 +20,10 @@ done
 
 darcs record -a -m add_foo | grep -i "finished recording"
 darcs dist
+darcs dist --zip
 
 mv temp1.tar.gz  ..
+mv temp1.zip  ..
 
 cd ..
 
@@ -32,4 +34,14 @@ tar xzf temp1.tar.gz
 
 diff -r temp_orig temp1
 
-rm -rf temp_orig temp1
+rm -rf temp1
+
+# Only do following if unzip is present
+unzip >& /dev/null &&
+(
+unzip temp1.zip
+diff -r temp_orig temp1
+rm -rf temp1
+)
+
+rm -rf temp_orig
