@@ -44,9 +44,6 @@ if [ -f "/usr/sbin/sendmail" -o -f "/sbin/sendmail" -o \
   exit 200
 fi
 
-DARCS_EDITOR=echo
-export DARCS_EDITOR
-
 mkdir temp1 temp2
 
 cd temp2
