@@ -23,7 +23,7 @@
 ## CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 ## SOFTWARE.
 
-. ../tests/lib                  # Load some portability helpers.
+. lib                  # Load some portability helpers.
 rm -rf R
 
 mkdir -p R/Foo/Bar		# 2 directory levels needed
