@@ -1,4 +1,4 @@
-set -ev
+. ./lib
 
 rm -rf temp1 temp2
 
