@@ -3,9 +3,6 @@
 
 # for issue381: "darcs send -o message --edit-description doesn't work"
 
-DARCS_EDITOR=echo
-export DARCS_EDITOR
-
 rm -rf temp1 temp2
 mkdir temp1 temp2
 
@@ -47,4 +44,3 @@ grep 'finished editing' bundle
 IFS=' ' darcs send --author=me -a --subject="it works" --to user@place.org --sendmail-command='grep "^Subject: it works$" %<' ../temp2
 
 cd ..
-rm -rf temp1 temp2
