@@ -2,8 +2,9 @@
 # Written in 2010 by Petr Rockai, placed in public domain
 
 . lib
+. httplib
 
-skip-formats darcs-1 # compressed repo is darcs-2
+only-format darcs-2 # compressed repo is darcs-2
 
 gunzip -c $TESTDATA/laziness-complete.tgz | tar xf -
 
@@ -15,7 +16,8 @@ test -e _darcs/packs/patches.tar.gz
 cd ..
 
 serve_http # sets baseurl
-darcs clone --packs $baseurl/laziness-complete S
+rm -rf S
+darcs clone --packs $baseurl/repo S
 cd S
 rm _darcs/prefs/sources # avoid any further contact with the original repository
 darcs check
