@@ -33,7 +33,7 @@ darcs init
 echo 'wibble' > wibble
 darcs rec -lam "add wibble"
 
-echo 'yy' | darcs rebase suspend | grep-once "Rebase in progress: 1 suspended patch"
+echo 'yy' | darcs rebase suspend 2>&1 | grep-once "Rebase in progress: 1 suspended patch"
 not test -f wibble
 echo 'yy' | darcs rebase unsuspend | grep-once "Rebase finished"
 test -f wibble
