@@ -13,14 +13,10 @@ darcs init
 darcs record -am foo --ask-deps | grep -i "Ok, if you don't want to record anything, that's fine!"
 
 # RT#476 - --ask-deps works when there are no patches
-if echo $OS | grep -i windows; then
-  echo This test does not work on Windows
-else
-  touch t.f
-  darcs add t.f
-  darcs record  -am add
-  echo a | darcs record  -am foo --ask-deps | grep -i 'finished recording'
-fi
+touch t.f
+darcs add t.f
+darcs record -am add
+echo a | darcs record  -am foo --ask-deps | grep -i 'finished recording'
 
 # RT#231 - special message is given for nonexistent directories
 not darcs record -am foo not_there.txt > log 2>&1
@@ -82,9 +78,7 @@ cd foo1
 darcs init
 echo zig > foo
 darcs add foo
-sleep 1
 darcs record -a -m add_foo -A x
-#sleep 1
 echo zag >> foo
 darcs record --ignore-time -a -m mod_foo -A x
 cd ../foo2
