@@ -24,8 +24,8 @@ darcs repair
 # remove each hashed pristine file one at a time
 for f in $(ls _darcs/pristine.hashed/*); do
   rm $f
-  not darcs check
-  darcs repair | grep -i "fixing pristine"
+  not darcs check --no-cache
+  darcs repair --no-cache | grep -i "fixing pristine"
 done
 
 cd ..
@@ -176,6 +176,6 @@ darcs mv d/f e/
 darcs record -am 'Move d/f to e/f.'
 rm _darcs/pristine.hashed/*     # Make the repository bogus
 cp -r _darcs archive
-not darcs check
+not darcs check --no-cache
 diff -r _darcs archive
 cd ..
