@@ -1,4 +1,4 @@
-#!/bin/env bash
+#!/usr/bin/env bash
 # A test for test --linear, test --bisect and test --backoff.
 # In general it construct various repositories and try
 # to find the last recent failing patch and match it with
@@ -13,7 +13,7 @@ if echo $OS | grep -i windows; then
 fi
 
 cp $TESTBIN/trackdown-bisect-helper.hs .
-ghc -o trackdown-bisect-helper trackdown-bisect-helper.hs
+ghc $GHC_FLAGS -o trackdown-bisect-helper trackdown-bisect-helper.hs
 
 function make_repo_with_test {
     rm -fr temp1
@@ -53,7 +53,6 @@ test_cmd='grep -q 1 j'
 # Section with test-cases
 #############################################################################
 
-# TEST01: Repo with success in the half
 testTrackdown() {
 make_repo_with_test $1
 if darcs test $test_args "$test_cmd" | is_found_good_patch $2; then
@@ -65,6 +64,17 @@ fi
 cleanup_repo_after
 }
 
+testTrackdownNoPasses() {
+make_repo_with_test $1
+if darcs test $test_args "$test_cmd" | grep "Noone passed the test"; then
+    echo "ok 1"
+else
+    echo "not ok 1. The trackdown should report 'Noone passed the test'."
+    exit 1
+fi
+cleanup_repo_after
+}
+
 # TEST01: Repo with success in the half
 test01() {
 testTrackdown '[1,1,0,0,0]' 3
@@ -72,7 +82,7 @@ testTrackdown '[1,1,0,0,0]' 3
 
 # TEST02: Repo without success condition
 test02() { 
-testTrackdown '[0,0,0,0,0]' 1
+testTrackdownNoPasses '[0,0,0,0,0]'
 }
 
 # TEST03: Repo with success condition at before last patch
